@@ -2,26 +2,11 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2015 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
 #include "lf_functions.h"
-#include "nisplus.h"
-
-/* We can't just compile this code and allow the library mechanism to omit the
-functions if they are not wanted, because we need to have the NIS+ header
-available for compiling. Therefore, compile these functions only if
-LOOKUP_NISPLUS is defined. However, some compilers don't like compiling empty
-modules, so keep them happy with a dummy when skipping the rest. Make it
-reference itself to stop picky compilers complaining that it is unused, and put
-in a dummy argument to stop even pickier compilers complaining about infinite
-loops. */
-
-#ifndef LOOKUP_NISPLUS
-static void dummy(int x) { dummy(x-1); }
-#else
-
 
 #include <rpcsvc/nis.h>
 
@@ -32,7 +17,7 @@ static void dummy(int x) { dummy(x-1); }
 
 /* See local README for interface description. */
 
-void *
+static void *
 nisplus_open(uschar *filename, uschar **errmsg)
 {
 return (void *)(1);    /* Just return something non-null */
@@ -56,9 +41,9 @@ name tagged on the end after a colon. If there is no result-field name, the
 yield is the concatenation of all the fields, preceded by their names and an
 equals sign. */
 
-int
+static int
 nisplus_find(void *handle, uschar *filename, uschar *query, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, uint *do_cache)
 {
 int i;
 int ssize = 0;
@@ -73,6 +58,8 @@ struct table_obj *ta;
 uschar *p = query + length;
 uschar *yield = NULL;
 
+do_cache = do_cache;   /* Placate picky compilers */
+
 /* Search backwards for a colon to see if a result field name
 has been given. */
 
@@ -207,7 +194,11 @@ field name, ensure the concatenated data is zero-terminated. */
 if (field_name != NULL)
   *errmsg = string_sprintf("NIS+ field %s not found for %s", field_name,
     query);
-else yield[offset] = 0;
+else
+  {
+  yield[offset] = 0;
+  store_reset(yield + offset + 1);
+  }
 
 /* Restore the colon in the query, and free result store before
 finishing. */
@@ -242,7 +233,7 @@ Arguments:
 Returns:     the processed string or NULL for a bad option
 */
 
-uschar *
+static uschar *
 nisplus_quote(uschar *s, uschar *opt)
 {
 int count = 0;
@@ -266,6 +257,41 @@ while (*s != 0)
 return quoted;
 }
 
-#endif  /* LOOKUP_NISPLUS */
+
+/*************************************************
+*         Version reporting entry point          *
+*************************************************/
+
+/* See local README for interface description. */
+
+#include "../version.h"
+
+void
+nisplus_version_report(FILE *f)
+{
+#ifdef DYNLOOKUP
+fprintf(f, "Library version: NIS+: Exim version %s\n", EXIM_VERSION_STR);
+#endif
+}
+
+
+static lookup_info _lookup_info = {
+  US"nisplus",                   /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+  nisplus_open,                  /* open function */
+  NULL,                          /* check function */
+  nisplus_find,                  /* find function */
+  NULL,                          /* no close function */
+  NULL,                          /* no tidy function */
+  nisplus_quote,                 /* quoting function */
+  nisplus_version_report         /* version reporting */
+};
+
+#ifdef DYNLOOKUP
+#define nisplus_lookup_module_info _lookup_module_info
+#endif
+
+static lookup_info *_lookup_list[] = { &_lookup_info };
+lookup_module_info nisplus_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 1 };
 
 /* End of lookups/nisplus.c */
